from __future__ import annotations
from typing import TYPE_CHECKING, Any, Callable, Generic, Iterable, Iterator, TypeVar, overload

T = TypeVar('T')
from .Track import Track

if TYPE_CHECKING:
    from Live.Base import Vector
    from Live.Chain import Chain
    from Live.LomObject import LomObject



class DeviceContainer(LomObject):
    """This class is a common super class of Track and Chain"""

    @property
    def _live_ptr(self) -> int:
        ...

class DeviceInsertMode(int):
    default: int = 0
    selected_left: int = 1
    selected_right: int = 2
    count: int = 3

class RoutingChannel:
    """This class represents a routing channel."""

    @property
    def display_name(self) -> str:
        """Display name of routing channel."""
        ...

    @property
    def layout(self) -> RoutingChannelLayout:
        """The routing channel's Layout, e.g., mono or stereo."""
        ...

class RoutingChannelLayout(int):
    midi: int = 0
    mono: int = 1
    stereo: int = 2

class RoutingChannelVector(Vector[RoutingChannel]):
    """A container for returning routing channels from Live."""

    def append(self, value: RoutingChannel | None, /) -> None:
        ...

    def extend(self, values: Iterable[RoutingChannel] | None, /) -> None:
        ...

class RoutingType:
    """This class represents a routing type."""

    @property
    def attached_object(self) -> Track:
        """Live object associated with the routing type."""
        ...

    @property
    def category(self) -> int:
        """Category of the routing type."""
        ...

    @property
    def display_name(self) -> str:
        """Display name of routing type."""
        ...

class RoutingTypeCategory(int):
    external: int = 0
    rewire: int = 1
    resampling: int = 2
    master: int = 3
    track: int = 4
    parent_group_track: int = 5
    none: int = 6
    invalid: int = 7

class RoutingTypeVector(Vector[RoutingType]):
    """A container for returning routing types from Live."""

    def append(self, value: RoutingType | None, /) -> None:
        ...

    def extend(self, values: Iterable[RoutingType] | None, /) -> None:
        ...

__all__ = ['Track', 'DeviceContainer', 'DeviceInsertMode', 'RoutingChannel', 'RoutingChannelLayout', 'RoutingChannelVector', 'RoutingType', 'RoutingTypeCategory', 'RoutingTypeVector']



from __future__ import annotations
from typing import TYPE_CHECKING, Any, Callable, Iterable
from .SimplerDevice import SimplerDevice

if TYPE_CHECKING:
    from Live.Base import IntVector



class PlaybackMode(int):
    classic: int = 0
    one_shot: int = 1
    slicing: int = 2

class SlicingPlaybackMode(int):
    mono: int = 0
    poly: int = 1
    thru: int = 2

def get_available_voice_numbers() -> IntVector:
    """Get a vector of valid Simpler voice numbers."""
    ...

__all__ = ['SimplerDevice', 'PlaybackMode', 'SlicingPlaybackMode', 'get_available_voice_numbers']



from __future__ import annotations
from typing import TYPE_CHECKING, Any, Callable, Generic, Iterable, Iterator, TypeVar, overload

T = TypeVar('T')
from .Clip import Clip

if TYPE_CHECKING:
    from Live.Base import Vector



class ClipLaunchQuantization(int):
    q_global: int = 0
    q_none: int = 1
    q_8_bars: int = 2
    q_4_bars: int = 3
    q_2_bars: int = 4
    q_bar: int = 5
    q_half: int = 6
    q_half_triplet: int = 7
    q_quarter: int = 8
    q_quarter_triplet: int = 9
    q_eighth: int = 10
    q_eighth_triplet: int = 11
    q_sixteenth: int = 12
    q_sixteenth_triplet: int = 13
    q_thirtysecond: int = 14

class GridQuantization(int):
    no_grid: int = 0
    g_8_bars: int = 1
    g_4_bars: int = 2
    g_2_bars: int = 3
    g_bar: int = 4
    g_half: int = 5
    g_quarter: int = 6
    g_eighth: int = 7
    g_sixteenth: int = 8
    g_thirtysecond: int = 9
    count: int = 10

class LaunchMode(int):
    trigger: int = 0
    gate: int = 1
    toggle: int = 2
    repeat: int = 3

class MidiNote:
    """An object representing a MIDI Note"""

    @property
    def duration(self) -> float:
        ...

    @duration.setter
    def duration(self, value: float) -> None: ...

    @property
    def mute(self) -> bool:
        ...

    @mute.setter
    def mute(self, value: bool) -> None: ...

    @property
    def note_id(self) -> int:
        """
        A numerical ID that's unique within the originating clip of the note. Not to be
        used directly, but important for other API calls, namely apply_note_modifications.
        """
        ...

    @property
    def pitch(self) -> int:
        ...

    @pitch.setter
    def pitch(self, value: int) -> None: ...

    @property
    def probability(self) -> float:
        ...

    @probability.setter
    def probability(self, value: float) -> None: ...

    @property
    def release_velocity(self) -> float:
        ...

    @release_velocity.setter
    def release_velocity(self, value: float) -> None: ...

    @property
    def start_time(self) -> float:
        ...

    @start_time.setter
    def start_time(self, value: float) -> None: ...

    @property
    def velocity(self) -> float:
        ...

    @velocity.setter
    def velocity(self, value: float) -> None: ...

    @property
    def velocity_deviation(self) -> float:
        ...

    @velocity_deviation.setter
    def velocity_deviation(self, value: float) -> None: ...

class MidiNoteSpecification:
    """
    An object specifying the data for creating a MIDI note. To be used with the
    add_new_notes function.
    """

    def __init__(self, pitch: int, start_time: float, duration: float, velocity: float = 100.0, mute: bool = False, probability: float = 1.0, velocity_deviation: float = 0.0, release_velocity: float = 64.0) -> None: ...

class MidiNoteVector(Vector[MidiNote]):
    """A container for holding MIDI notes from Live."""

    def append(self, value: MidiNote | None, /) -> None:
        ...

    def extend(self, values: Iterable[MidiNote] | None, /) -> None:
        ...

class WarpMarker:
    """This class represents a WarpMarker type."""

    def __init__(self, sample_time: float, beat_time: float) -> None: ...

    @property
    def beat_time(self) -> float:
        """A WarpMarker's beat time."""
        ...

    @property
    def sample_time(self) -> float:
        """A WarpMarker's sample time."""
        ...

class WarpMarkerVector(Vector[WarpMarker]):
    """A container for returning warp markers from Live."""

    def append(self, value: WarpMarker | None, /) -> None:
        ...

    def extend(self, values: Iterable[WarpMarker] | None, /) -> None:
        ...

class WarpMode(int):
    beats: int = 0
    tones: int = 1
    texture: int = 2
    repitch: int = 3
    complex: int = 4
    rex: int = 5
    complex_pro: int = 6
    count: int = 7

__all__ = ['Clip', 'ClipLaunchQuantization', 'GridQuantization', 'LaunchMode', 'MidiNote', 'MidiNoteSpecification', 'MidiNoteVector', 'WarpMarker', 'WarpMarkerVector', 'WarpMode']



from __future__ import annotations
from typing import TYPE_CHECKING, Any, Callable, Iterable

if TYPE_CHECKING:
    from . import MessageButtons, PushDialogType, UnavailableFeatureVector, Variants
    from Live.Base import StringVector, Text, Vector
    from Live.Browser import Browser
    from Live.LomObject import LomObject
    from Live.Song import Song



class Application(LomObject):
    """This class represents the Live application."""

    class View(LomObject):
        """This class represents the view aspects of the Live application."""

        class NavDirection(int):
            up: int = 0
            down: int = 1
            left: int = 2
            right: int = 3

        @property
        def _live_ptr(self) -> int:
            ...

        def add_browse_mode_listener(self, callback: Callable | None, /) -> None:
            """
            Add a listener function or method, which will be called as soon as the
            property "browse_mode" has changed.
            """
            ...

        def add_focused_document_view_listener(self, callback: Callable | None, /) -> None:
            """
            Add a listener function or method, which will be called as soon as the
            property "focused_document_view" has changed.
            """
            ...

        def add_is_view_visible_listener(self, view_name: str | None, callback: Callable | None, /) -> None:
            """
            Add a listener function or method, which will be called as soon as the
            property "is_view_visible" has changed.
            """
            ...

        def add_view_focus_changed_listener(self, callback: Callable | None, /) -> None:
            """
            Add a listener function or method, which will be called as soon as the
            property "view_focus_changed" has changed.
            """
            ...

        def available_main_views(self) -> StringVector:
            """
            Return a list of strings with the available subcomponent views, which
            is to be specified, when using the rest of this classes functions.
            A 'subcomponent view' is a main view component of a document view, like
            the Session view, the Arranger or Detailview and so on...
            """
            ...

        @property
        def browse_mode(self) -> bool:
            """Return true if HotSwap mode is active for any target."""
            ...

        def browse_mode_has_listener(self, callback: Callable | None, /) -> bool:
            """
            Returns true, if the given listener function or method is connected
            to the property "browse_mode".
            """
            ...

        @property
        def canonical_parent(self) -> Application:
            """Get the canonical parent of the application view."""
            ...

        def focus_view(self, view: str | None, /) -> None:
            """Show and focus one through the identifier string specified view."""
            ...

        @property
        def focused_document_view(self) -> str:
            """
            Return the name of the document view ('Session' or 'Arranger')
            shown in the currently selected window.
            """
            ...

        def focused_document_view_has_listener(self, callback: Callable | None, /) -> bool:
            """
            Returns true, if the given listener function or method is connected
            to the property "focused_document_view".
            """
            ...

        def hide_view(self, view_name: str | None, /) -> None:
            """Hide one through the identifier string specified view."""
            ...

        def is_view_visible(self, identifier: str | None, main_window_only: bool = True, /) -> bool:
            """
            Return true if the through the identifier string specified view is currently
            visible. If main_window_only is set to False, this will also check in second
            window. Notifications from the second window are not yet supported.
            """
            ...

        def is_view_visible_has_listener(self, view_name: str | None, callback: Callable | None, /) -> bool:
            """
            Returns true, if the given listener function or method is connected
            to the property "is_view_visible".
            """
            ...

        def remove_browse_mode_listener(self, callback: Callable | None, /) -> None:
            """
            Remove a previously set listener function or method from
            property "browse_mode".
            """
            ...

        def remove_focused_document_view_listener(self, callback: Callable | None, /) -> None:
            """
            Remove a previously set listener function or method from
            property "focused_document_view".
            """
            ...

        def remove_is_view_visible_listener(self, view_name: str | None, callback: Callable | None, /) -> None:
            """
            Remove a previously set listener function or method from
            property "is_view_visible".
            """
            ...

        def remove_view_focus_changed_listener(self, callback: Callable | None, /) -> None:
            """
            Remove a previously set listener function or method from
            property "view_focus_changed".
            """
            ...

        def scroll_view(self, direction: int | None, view_name: str | None, modifier_pressed: bool | None, /) -> None:
            """
            Scroll through the identifier string specified view into the given
            direction, if possible. Will silently return if the specified view
            can not perform the requested action.
            """
            ...

        def show_view(self, view: str | None, /) -> None:
            """
            Show one through the identifier string specified view. Will throw a
            runtime error if this is called in Live's initialization scope.
            """
            ...

        def toggle_browse(self) -> None:
            """
            Reveals the device chain, the browser and starts hot swap for
            the selected device. Calling this function again stops hot swap.
            """
            ...

        def view_focus_changed_has_listener(self, callback: Callable | None, /) -> bool:
            """
            Returns true, if the given listener function or method is connected
            to the property "view_focus_changed".
            """
            ...

        def zoom_view(self, direction: int | None, view_name: str | None, modifier_pressed: bool | None, /) -> None:
            """
            Zoom through the identifier string specified view into the given
            direction, if possible. Will silently return if the specified view
            can not perform the requested action.
            """
            ...

    def add_average_process_usage_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "average_process_usage" has changed.
        """
        ...

    def add_control_surfaces_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "control_surfaces" has changed.
        """
        ...

    def add_open_dialog_count_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "open_dialog_count" has changed.
        """
        ...

    def add_peak_process_usage_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "peak_process_usage" has changed.
        """
        ...

    def add_unavailable_features_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "unavailable_features" has changed.
        """
        ...

    @property
    def average_process_usage(self) -> float:
        """Reports Live's average CPU load."""
        ...

    def average_process_usage_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "average_process_usage".
        """
        ...

    @property
    def browser(self) -> Browser:
        """Returns an interface to the browser."""
        ...

    @property
    def canonical_parent(self) -> None:
        """Returns the canonical parent of the application."""
        ...

    @property
    def control_surfaces(self) -> Vector[object]:
        """
        Const access to a list of the control surfaces selected in preferences, in the same order.
        The list contains None if no control surface is active at that index.
        """
        ...

    def control_surfaces_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "control_surfaces".
        """
        ...

    @property
    def current_dialog_button_count(self) -> int:
        """Number of buttons on the current dialog."""
        ...

    @property
    def current_dialog_message(self) -> str:
        """Text of the last dialog that appeared; Empty if all dialogs just disappeared."""
        ...

    def get_bugfix_version(self) -> int:
        """Returns an integer representing the bugfix version of Live."""
        ...

    def get_build_id(self) -> str:
        """Returns a string identifying the build."""
        ...

    def get_document(self) -> Song:
        """Returns the current Live Set."""
        ...

    def get_major_version(self) -> int:
        """Returns an integer representing the major version of Live."""
        ...

    def get_minor_version(self) -> int:
        """Returns an integer representing the minor version of Live."""
        ...

    def get_variant(self) -> str:
        """Returns one of the strings in Live.Application.Variants."""
        ...

    def get_version_string(self) -> str:
        """Returns the full version string of Live."""
        ...

    def has_option(self, option_name: str | None, /) -> bool:
        """Returns True if the given entry exists in Options.txt, False otherwise."""
        ...

    @property
    def number_of_push_apps_running(self) -> int:
        """Returns the number of connected Push apps."""
        ...

    @property
    def open_dialog_count(self) -> int:
        """The number of open dialogs in Live. 0 if not dialog is open."""
        ...

    def open_dialog_count_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "open_dialog_count".
        """
        ...

    @property
    def peak_process_usage(self) -> float:
        """Reports Live's peak CPU load."""
        ...

    def peak_process_usage_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "peak_process_usage".
        """
        ...

    def press_current_dialog_button(self, index: int | None, /) -> None:
        """Press a button, by index, on the current message box."""
        ...

    def remove_average_process_usage_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "average_process_usage".
        """
        ...

    def remove_control_surfaces_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "control_surfaces".
        """
        ...

    def remove_open_dialog_count_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "open_dialog_count".
        """
        ...

    def remove_peak_process_usage_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "peak_process_usage".
        """
        ...

    def remove_unavailable_features_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "unavailable_features".
        """
        ...

    def show_message(self, text: Text | None, buttons: MessageButtons | int = 0, enable_markup: bool = False, show_success_icon: bool = False, /) -> int:
        """Shows a message box, returning the position of the pressed button."""
        ...

    def show_on_the_fly_message(self, message: str | None, buttons: MessageButtons | int = 0, enable_markup: bool = False, show_success_icon: bool = False, push_dialog_type: PushDialogType | int = 0, /) -> int:
        """Same as show_message, but for when there is no predefined Text object."""
        ...

    @property
    def unavailable_features(self) -> UnavailableFeatureVector:
        """List of features that are unavailable due to limitations of the current Live edition."""
        ...

    def unavailable_features_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "unavailable_features".
        """
        ...

    @property
    def view(self) -> View:
        """Returns the applications view component."""
        ...

__all__ = ['Application']



from __future__ import annotations
from typing import TYPE_CHECKING, Any, Callable, Generic, Iterable, Iterator, TypeVar, overload

T = TypeVar('T')
from .Device import Device

if TYPE_CHECKING:
    from Live.Base import Vector
    from Live.DeviceParameter import DeviceParameter



class ATimeableValueVector(Vector[DeviceParameter]):

    def append(self, value: DeviceParameter | None, /) -> None:
        ...

    def extend(self, values: Iterable[DeviceParameter] | None, /) -> None:
        ...

class DeviceType(int):
    """The type of the device."""
    undefined: int = 0
    instrument: int = 1
    audio_effect: int = 2
    midi_effect: int = 4

__all__ = ['Device', 'ATimeableValueVector', 'DeviceType']



from __future__ import annotations
from typing import TYPE_CHECKING, Any, Callable, Iterable

if TYPE_CHECKING:
    from . import EnvelopeEventVector
    from Live.Clip import Clip
    from Live.LomObject import LomObject



class Envelope(LomObject):
    """This class represents an automation or modulation envelope in Live."""

    @property
    def _live_ptr(self) -> int:
        ...

    @property
    def canonical_parent(self) -> Clip:
        """Get the canonical parent of the envelope."""
        ...

    def delete_events_in_range(self, start_time: float | None, end_time: float | None, /) -> None:
        """Deletes the events in the specified time range."""
        ...

    def events_in_range(self, start_time: float | None, end_time: float | None, /) -> EnvelopeEventVector:
        """Returns the events in the specified time range."""
        ...

    def insert_step(self, start_time: float | None, length: float | None, value: float | None, /) -> None:
        """Given a start time, a step length and a value, creates a step in the envelope."""
        ...

    def value_at_time(self, time: float | None, /) -> float:
        """Returns the parameter value at the specified time."""
        ...

__all__ = ['Envelope']



from __future__ import annotations
from typing import TYPE_CHECKING, Any, Callable, Iterable
from .DeviceParameter import DeviceParameter


class AutomationState(int):
    none: int = 0
    playing: int = 1
    overridden: int = 2

class ParameterState(int):
    enabled: int = 0
    irrelevant: int = 1
    disabled: int = 2

__all__ = ['DeviceParameter', 'AutomationState', 'ParameterState']



from __future__ import annotations
from typing import TYPE_CHECKING, Any, Callable, Iterable
from .Eq8Device import Eq8Device


class EditMode(int):
    a: int = 0
    b: int = 1

class GlobalMode(int):
    stereo: int = 0
    left_right: int = 1
    mid_side: int = 2

__all__ = ['Eq8Device', 'EditMode', 'GlobalMode']



from __future__ import annotations
from typing import TYPE_CHECKING, Any, Callable, Iterable
from .Groove import Groove


class Base(int):
    gb_four: int = 0
    gb_eight: int = 1
    gb_eight_triplet: int = 2
    gb_sixteen: int = 3
    gb_sixteen_triplet: int = 4
    gb_thirtytwo: int = 5
    count: int = 6

__all__ = ['Groove', 'Base']



from __future__ import annotations
from typing import TYPE_CHECKING, Any, Callable, Generic, Iterable, Iterator, TypeVar, overload

T = TypeVar('T')
from .Application import Application

if TYPE_CHECKING:
    from Live.Base import Vector



class ControlDescription:
    """Describes a control present in a control surface proxy"""

    def __init__(self) -> None: ...

    @property
    def id(self) -> int:
        ...

    @property
    def name(self) -> str:
        ...

class ControlDescriptionVector(Vector[ControlDescription]):
    """A container for returning control descriptions."""

    def append(self, value: ControlDescription | None, /) -> None:
        ...

    def extend(self, values: Iterable[ControlDescription] | None, /) -> None:
        ...

class ControlSurfaceProxy:
    """Represents a control surface running in a different process. For use by M4L"""

    def add_control_values_arrived_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "control_values_arrived" has changed.
        """
        ...

    def add_midi_received_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "midi_received" has changed.
        """
        ...

    def add_pad_layout_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "pad_layout" has changed.
        """
        ...

    @property
    def control_descriptions(self) -> ControlDescriptionVector:
        ...

    def control_values_arrived_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "control_values_arrived".
        """
        ...

    def enable_receive_midi(self, enabled: bool | None, /) -> None:
        ...

    def fetch_received_midi_messages(self) -> tuple[tuple[int, ...], ...]:
        ...

    def fetch_received_values(self) -> tuple[tuple[int, Any], ...]:
        ...

    def grab_control(self, control: int | None, /) -> None:
        ...

    def midi_received_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "midi_received".
        """
        ...

    @property
    def pad_layout(self) -> str:
        """The layout of pads on Push."""
        ...

    def pad_layout_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "pad_layout".
        """
        ...

    def release_control(self, control: int | None, /) -> None:
        ...

    def remove_control_values_arrived_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "control_values_arrived".
        """
        ...

    def remove_midi_received_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "midi_received".
        """
        ...

    def remove_pad_layout_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "pad_layout".
        """
        ...

    def send_midi(self, midi_event_bytes: tuple[int, ...] | None, /) -> None:
        ...

    def send_value(self, value: tuple[Any, ...] | None, /) -> None:
        ...

    def subscribe_to_control(self, control: int | None, /) -> None:
        ...

    @property
    def type_name(self) -> str:
        ...

    def unsubscribe_from_control(self, control: int | None, /) -> None:
        ...

class MessageButtons(int):
    """Specifies the characteristics of the message box, e.g. which buttons to show."""
    OK_BUTTON: int = 0
    OK_NEW_SET_BUTTON: int = 1
    OK_RETRY_BUTTON: int = 2
    SAVE_DONT_SAVE_BUTTON: int = 3
    OK_ACCOUNT_BUTTON: int = 4
    OK_PURCHASE_BUTTON: int = 5

class PushDialogType(int):
    """Specifies the dialog type for Push."""
    MESSAGE_BOX: int = 0
    OUT_OF_UNLOCKS_DIALOG: int = 5
    RENT_TO_OWN_LICENSE_EXPIRED_DIALOG: int = 7

class UnavailableFeature(int):
    note_velocity_ranges_and_probabilities: int = 0

class UnavailableFeatureVector(Vector[UnavailableFeature]):
    """A container for returning unavailable features."""

    def append(self, value: UnavailableFeature | None, /) -> None:
        ...

    def extend(self, values: Iterable[UnavailableFeature] | None, /) -> None:
        ...

class Variants:
    """Holds strings representing what type of Live is running."""
    BETA: str = "Beta"
    INTRO: str = "Intro"
    LITE: str = "Lite"
    STANDARD: str = "Standard"
    SUITE: str = "Suite"
    TRIAL: str = "Trial"

def combine_apcs() -> bool:
    """Returns true if multiple APCs should be combined."""
    ...

def encrypt_challenge(dongle1: int | None, dongle2: int | None, key_index: int = 0, /) -> tuple[int, ...]:
    """Returns an encrypted challenge based on the TEA algortithm"""
    ...

def encrypt_challenge2(challenge: int | None, /) -> int:
    """Returns the UMAC hash for the given challenge."""
    ...

def get_application() -> Application:
    """Returns the application instance."""
    ...

def get_random_int(min_value: int | None, max_value: int | None, /) -> int:
    """Returns a random integer from the given range."""
    ...

__all__ = ['Application', 'ControlDescription', 'ControlDescriptionVector', 'ControlSurfaceProxy', 'MessageButtons', 'PushDialogType', 'UnavailableFeature', 'UnavailableFeatureVector', 'Variants', 'combine_apcs', 'encrypt_challenge', 'encrypt_challenge2', 'get_application', 'get_random_int']



from __future__ import annotations
from typing import TYPE_CHECKING, Any, Callable, Iterable
from .TuningSystem import TuningSystem


class PitchClassAndOctave:
    """This class represents a PitchClassAndOctave type."""

    def __init__(self, index_in_octave: int, octave: int) -> None: ...

    @property
    def index_in_octave(self) -> int:
        """A PitchClassAndOctave's index within the pseudo octave."""
        ...

    @property
    def octave(self) -> int:
        """A PitchClassAndOctave's octave."""
        ...

class ReferencePitch:
    """This class represents a ReferencePitch type."""

    def __init__(self, index_in_octave: int, octave: int, frequency: float) -> None: ...

    @property
    def frequency(self) -> float:
        """A ReferencePitch's frequency in Hz."""
        ...

    @property
    def index_in_octave(self) -> int:
        """A ReferencePitch's index within the pseudo octave."""
        ...

    @property
    def octave(self) -> int:
        """A ReferencePitch's octave."""
        ...

__all__ = ['TuningSystem', 'PitchClassAndOctave', 'ReferencePitch']



from __future__ import annotations
from typing import TYPE_CHECKING, Any, Callable, Generic, Iterable, Iterator, TypeVar, overload

T = TypeVar('T')
from .Browser import Browser

if TYPE_CHECKING:
    from Live.Base import Vector



class BrowserItem:
    """This class represents an item of the browser hierarchy."""

    @property
    def children(self) -> BrowserItemVector:
        """Const access to the descendants of this browser item."""
        ...

    @property
    def is_device(self) -> bool:
        """Indicates if the browser item represents a device."""
        ...

    @property
    def is_folder(self) -> bool:
        """Indicates if the browser item represents folder."""
        ...

    @property
    def is_loadable(self) -> bool:
        """True if item can be loaded via the Browser's 'load_item' method."""
        ...

    @property
    def is_selected(self) -> bool:
        """True if the item is ancestor of or the actual selection."""
        ...

    @property
    def iter_children(self) -> BrowserItemIterator:
        """Const iterable access to the descendants of this browser item."""
        ...

    @property
    def name(self) -> str:
        """Const access to the canonical display name of this browser item."""
        ...

    @property
    def source(self) -> str:
        """Specifies where does item come from -- i.e. Live pack, user library..."""
        ...

    @property
    def uri(self) -> str:
        """The uri describes a unique identifier for a browser item."""
        ...

class BrowserItemIterator(Iterable[BrowserItem]):
    """This class iterates over children of another BrowserItem."""

class BrowserItemVector(Vector[BrowserItem]):
    """A container for returning browser items from Live."""

    def append(self, value: BrowserItem | None, /) -> None:
        ...

    def extend(self, values: Iterable[BrowserItem] | None, /) -> None:
        ...

class FilterType(int):
    disabled: int = -1
    hotswap_off: int = 0
    instrument_hotswap: int = 1
    audio_effect_hotswap: int = 2
    midi_effect_hotswap: int = 3
    drum_pad_hotswap: int = 4
    midi_track_devices: int = 5
    samples: int = 6
    count: int = 7

class Relation(int):
    ancestor: int = 0
    equal: int = 1
    descendant: int = 2
    none: int = 3

__all__ = ['Browser', 'BrowserItem', 'BrowserItemIterator', 'BrowserItemVector', 'FilterType', 'Relation']



from __future__ import annotations
from typing import TYPE_CHECKING, Any, Callable, Iterable
from .ClipSlot import ClipSlot


class ClipSlotPlayingState(int):
    stopped: int = 0
    started: int = 1
    recording: int = 2

__all__ = ['ClipSlot', 'ClipSlotPlayingState']



from __future__ import annotations
from typing import TYPE_CHECKING, Any, Callable, Iterable
from .WavetableDevice import WavetableDevice


class EffectMode(int):
    none: int = 0
    frequency_modulation: int = 1
    sync_and_pulse_width: int = 2
    warp_and_fold: int = 3

class FilterRouting(int):
    serial: int = 0
    parallel: int = 1
    split: int = 2

class ModulationSource(int):
    amp_envelope: int = 0
    envelope_2: int = 1
    envelope_3: int = 2
    lfo_1: int = 3
    lfo_2: int = 4
    midi_velocity: int = 5
    midi_note: int = 6
    midi_pitch_bend: int = 7
    midi_channel_pressure: int = 8
    midi_mod_wheel: int = 9
    midi_random: int = 10

class UnisonMode(int):
    none: int = 0
    classic: int = 1
    slow_shimmer: int = 2
    fast_shimmer: int = 3
    phase_sync: int = 4
    position_spread: int = 5
    random_note: int = 6

class VoiceCount(int):
    two: int = 0
    three: int = 1
    four: int = 2
    five: int = 3
    six: int = 4
    seven: int = 5
    eight: int = 6

class Voicing(int):
    mono: int = 0
    poly: int = 1

__all__ = ['WavetableDevice', 'EffectMode', 'FilterRouting', 'ModulationSource', 'UnisonMode', 'VoiceCount', 'Voicing']



from __future__ import annotations
from typing import TYPE_CHECKING, Any, Callable, Iterable
from .Sample import Sample


class SlicingBeatDivision(int):
    sixteenth: int = 0
    sixteenth_triplett: int = 1
    eighth: int = 2
    eighth_triplett: int = 3
    quarter: int = 4
    quarter_triplett: int = 5
    half: int = 6
    half_triplett: int = 7
    one_bar: int = 8
    two_bars: int = 9
    four_bars: int = 10

class SlicingStyle(int):
    transient: int = 0
    beat: int = 1
    region: int = 2
    manual: int = 3

class TransientLoopMode(int):
    off: int = 0
    forward: int = 1
    alternate: int = 2

__all__ = ['Sample', 'SlicingBeatDivision', 'SlicingStyle', 'TransientLoopMode']



from __future__ import annotations
from typing import TYPE_CHECKING, Any, Callable, Iterable
from .Song import Song

if TYPE_CHECKING:
    from Live.LomObject import LomObject



class BeatTime:
    """Represents a Time, splitted into Bars, Beats, SubDivision and Ticks."""

    def __init__(self) -> None: ...

    @property
    def bars(self) -> int:
        ...

    @bars.setter
    def bars(self, value: int) -> None: ...

    @property
    def beats(self) -> int:
        ...

    @beats.setter
    def beats(self, value: int) -> None: ...

    @property
    def sub_division(self) -> int:
        ...

    @sub_division.setter
    def sub_division(self, value: int) -> None: ...

    @property
    def ticks(self) -> int:
        ...

    @ticks.setter
    def ticks(self, value: int) -> None: ...

class CaptureDestination(int):
    """The destination for MIDI capture."""
    auto: int = 0
    session: int = 1
    arrangement: int = 2

class CaptureMode(int):
    """The capture mode that is used for capture and insert scene."""
    all: int = 0
    all_except_selected: int = 1

class CuePoint(LomObject):
    """Represents a 'Marker' in the arrangement."""

    @property
    def _live_ptr(self) -> int:
        ...

    def add_name_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "name" has changed.
        """
        ...

    def add_time_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "time" has changed.
        """
        ...

    @property
    def canonical_parent(self) -> Song:
        """Get the canonical parent of the cue point."""
        ...

    def jump(self) -> None:
        """
        When the Song is playing, set the playing-position quantized to
        this Cuepoint's time. When not playing, simply move the start
        playing position.
        """
        ...

    @property
    def name(self) -> str:
        """Get/Set/Listen to the name of this CuePoint, as visible in the arranger."""
        ...

    @name.setter
    def name(self, value: str) -> None: ...

    def name_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "name".
        """
        ...

    def remove_name_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "name".
        """
        ...

    def remove_time_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "time".
        """
        ...

    @property
    def time(self) -> float:
        """Get/Listen to the CuePoint's time in beats."""
        ...

    def time_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "time".
        """
        ...

class Quantization(int):
    q_no_q: int = 0
    q_8_bars: int = 1
    q_4_bars: int = 2
    q_2_bars: int = 3
    q_bar: int = 4
    q_half: int = 5
    q_half_triplet: int = 6
    q_quarter: int = 7
    q_quarter_triplet: int = 8
    q_eight: int = 9
    q_eight_triplet: int = 10
    q_sixtenth: int = 11
    q_sixtenth_triplet: int = 12
    q_thirtytwoth: int = 13

class RecordingQuantization(int):
    rec_q_no_q: int = 0
    rec_q_quarter: int = 1
    rec_q_eight: int = 2
    rec_q_eight_triplet: int = 3
    rec_q_eight_eight_triplet: int = 4
    rec_q_sixtenth: int = 5
    rec_q_sixtenth_triplet: int = 6
    rec_q_sixtenth_sixtenth_triplet: int = 7
    rec_q_thirtysecond: int = 8

class SessionRecordStatus(int):
    off: int = 0
    on: int = 1
    transition: int = 2

class SmptTime:
    """
    Represents a Time, split into Hours, Minutes, Seconds and Frames.
    The frame type must be specified when calling a function that returns
    a SmptTime.
    """

    def __init__(self) -> None: ...

    @property
    def frames(self) -> int:
        ...

    @frames.setter
    def frames(self, value: int) -> None: ...

    @property
    def hours(self) -> int:
        ...

    @hours.setter
    def hours(self, value: int) -> None: ...

    @property
    def minutes(self) -> int:
        ...

    @minutes.setter
    def minutes(self, value: int) -> None: ...

    @property
    def seconds(self) -> int:
        ...

    @seconds.setter
    def seconds(self, value: int) -> None: ...

class TimeFormat(int):
    ms_time: int = 0
    smpte_24: int = 1
    smpte_25: int = 2
    smpte_30: int = 3
    smpte_30_drop: int = 4
    smpte_29: int = 5

def get_all_scales_ordered() -> tuple[tuple[str, tuple[int, ...]], ...]:
    """Get an ordered tuple of tuples of all available scale names to intervals."""
    ...

__all__ = ['Song', 'BeatTime', 'CaptureDestination', 'CaptureMode', 'CuePoint', 'Quantization', 'RecordingQuantization', 'SessionRecordStatus', 'SmptTime', 'TimeFormat', 'get_all_scales_ordered']
